*! ram 07feb2011
* fixed to work when ll is not reported by mim because it does
* not vary across H0 models
* only works with mim v.1.1.1

* ram 24nov2008
* added error message with survey commands

* ram 10nov2008
* Meng confirmed that there is a misprint, and that Li et al. is the appropriate denom. df
* ram 3nov2008 version 1.0
* ram 31oct2008 changed notation (to Meng and Rubin), added r_L to the output
* ram 11sept2008 works with logit, ologit, and reg
* ram 05sept2008

program define milrtest, rclass
    version 9
    syntax varlist
* no if or in, adopts if and in from mim.

unab test : `varlist'


*** Check for errors/compatability ****
if("`e(MIM_prefix2)'"!="mim"){
    display as error "The most recent estimation command run was not mim."
    error 498
}
if("`e(MIM_cmd)'"!="logit"&"`e(MIM_cmd)'"!="logistic"& ///
    "`e(MIM_cmd)'"!="regress"&"`e(MIM_cmd)'"!="ologit"){
    display as error "Estimation command `e(MIM_cmd)' not supported"
    error 498
}

if("`e(MIM_prefix)'"=="svy"){
    display as error "LR test likely invalid for models with robust vce"
    error 498
}

if("`e(MIM_1_wtype)'"=="pweight"){
        display as error "LR test invalid in case of pweighted estimators"
        error 498
    }

* select command type for constrained likelihood
if("`e(MIM_cmd)'"=="logit"|"`e(MIM_cmd)'"=="logistic"){
    local cmd logit
}
* change this to just use the value of `command'
if("`e(MIM_cmd)'"=="regress"|"`e(MIM_cmd)'"=="reg"){
    local cmd reg
}
if("`e(MIM_cmd)'"=="ologit"){
    local cmd ologit
}


* Collect info from last run of MIM (H1)
local dep = e(MIM_depvar)
local command = e(MIM_cmd)
local mjlevels =  e(MIM_levels)
local m = e(MIM_m)

local h1_IVs = e(MIM_1_cmdline)
local h1_IVs : list local(h1_IVs) - local(dep)
local h1_IVs : list local(h1_IVs) - local(command)
local t = "version 9, missing :"
local h1_IVs : list local(h1_IVs) - local(t)

* get command line for later on 
local cmd_line = e(MIM_1_cmdline)
local cmd_line :  list local(cmd_line) - local(t)

* list of H1 IVs for reg and logit
if("`command'"=="regress"|"`command'"=="logit"|"`command'"=="logistic"){
    mat h1_B = e(MIM_Q)
    return matrix h1_coefs = h1_B, copy
    if(`=colsof(h1_B)'!=`=`: list sizeof local(h1_IVs)'+1'){
        display as error "Number of h1 independent variables does not" ///
            " match number of h1 betas." _newline ///
            as text "This may be due to some variables dropped from" ///
            " estimation command, check mim output carefully."
        error 498
    }
}
* list of H1 IVs for ologit
if("`command'"=="ologit"){
    mat coefs = e(MIM_Q)
    return matrix h1_coefs = coefs, copy
    mat h1_B = coefs[1,1..`: list sizeof local(h1_IVs)']
    mat h1_cuts = coefs[1,`=`: list sizeof local(h1_IVs)'+1'...]
    local h1_cuts cuts(h1_cuts)
    local cats cats(`e(MIM_1_k_cat)')
    if(`=colsof(h1_B)'!=`: list sizeof local(h1_IVs)'){
        display as error "Number of h1 independent variables does not" ///
            " match number of h1 betas." _newline ///
            as text "This may be due to some variables dropped from" ///
            " estimation command, check mim output carefully."
        error 498
    }
}

* get the log likelihood for the fully unconstrained (H1) model from MIM
foreach mj of local mjlevels {
    local h1_uc_`mj' = e(MIM_`mj'_ll)
}

* are all variables in test also in the hi_IVs list?
if(`: list local(test) in local(h1_IVs)'!=1){
    local error : list local(test) - local(h1_IVs)
    display as error "`error' not found"
    error 498
}

local h0_IVs : list local(h1_IVs) - local(test)
local r = `: list sizeof local(test)'

* di "H0 iVs: `h0_IVs'"

* run the command to return the sample
 quietly: `cmd_line ' if(_mj!=0)
tempvar insample
gen `insample' = e(sample)

* RUN mim for H0
quietly: mim: `command' `dep' `h0_IVs'
* list of H0 IVs for reg and logit
if("`command'"=="regress"|"`command'"=="logit"|"`command'"=="logistic"){
    mat h0_B = e(MIM_Q)
    return matrix h0_coefs = h0_B, copy
}

* list of H0 IVs for ologit
if("`command'"=="ologit"){
    mat coefs = e(MIM_Q)
    return matrix h0_coefs = coefs, copy
    if(`: list sizeof local(h0_IVs)'>0){
        mat h0_B = coefs[1,1..`: list sizeof local(h0_IVs)']
    }
    mat h0_cuts = coefs[1,`=`: list sizeof local(h0_IVs)'+1'...]
    local h0_cuts cuts(h0_cuts)
}

* get the log likelihood for the fully unconstrained h0 model from mim
local ind = 0
foreach mj of local mjlevels {
    local h0_uc_`mj' = e(MIM_`mj'_ll)
	if("`h0_uc_`mj''"==".") {
		local h0_uc_`mj' = e(MIM_1_ll)
		local ind = 1
	}
}
if(`ind'==1) {
	display "Warning, the final log-likelihood of the null model does not vary" _n ///
	"most likely because the values of variables do not vary across imputations." _n ///
	"The model is:" _n "`command' `dep' `h0_IVs'"
}


foreach mj of local mjlevels {

*        Calculate LL for h1 with estimates constrained.       *
        constrained_ll_`cmd', l_insample(`insample')  ///
            l_mj(`mj') l_IVs(`h1_IVs') l_dep(`dep') betas(h1_B) `cats' `h1_cuts'
            * l_markvar(`markvar')

            local h1_c_`mj' = ll



*        Calculate LL for h0 with estimates constrained.       *
        constrained_ll_`cmd' , l_insample(`insample')  ///
            l_mj(`mj') l_IVs(`h0_IVs') l_dep(`dep') betas(h0_B) `cats' `h0_cuts'
            * l_markvar(`markvar')

        local h0_c_`mj' = ll

}

* DROP COEFFICIENT MATRICES
capture: mat drop h0_cuts
capture: mat drop coefs
capture: mat drop h1_cuts
capture: mat drop h1_B
capture: mat drop h0_B


* calculate test statistics
* r is the number of restrictions
* M is the number of imputed datasets
* you also need to calculate DF
local k = `r'
local d_L = 0
local d_m= 0
foreach mj of local mjlevels {
    local d_L = `d_L' + 2*abs(`h1_c_`mj''-`h0_c_`mj'')
    local d_m = `d_m' + 2*abs(`h1_uc_`mj''-`h0_uc_`mj'')
}

local d_L = `d_L' / `m'
local d_m = `d_m'/`m'

* 3.7 and 3.8 from Meng and Rubin
local r_L = ((`m'+1)/(`k'*(`m'-1 )))*(`d_m'-`d_L')
local lr_test_stat = `d_L'/(`k'*(1+`r_L'))


* degrees of freedom
* test statistic follows an F distribution with df:
* numerator df = r
* denominator df is calculated below

local v = `k'*(`m'-1)

* 1.8 from Li et al (Meng and Rubin 2.7 contains a misprint)
if(`v'>4){
    local ddf = 4 + (`v'-4)*[1+(1-(2/`v'))*`r_L'^(-1)]^2
}
* 2.7 From Meng and Rubin
if(`v'<=4){
    local ddf = .5*`v'*(1+(1/`k'))*(1+`r_L'^(-1))^2
}
local pvalue = `=Ftail(`r',`ddf',`lr_test_stat')'


* return results
return scalar m = `m'
return scalar test_stat = `lr_test_stat'
return scalar df_n = `r'
return scalar df_d = `ddf'
return scalar p = `pvalue'
* mean of LR test from the m imputed datasets -- constrained model
return scalar d_L = `d_L'
* mean of LR tests from m imputed datasets -- unconstrained model
return scalar d_m = `d_m'
return scalar r_L = `r_L'

return local h1_model "`command' `dep' `h1_IVs'"
return local h0_model  "`command' `dep' `h0_IVs'"
return local cmd `command'

foreach mj of local mjlevels {
    return scalar h1_uc_`mj' = `h1_uc_`mj''
    return scalar h0_uc_`mj' = `h0_uc_`mj''
    return scalar h1_c_`mj' = `h1_c_`mj''
    return scalar h0_c_`mj' = `h0_c_`mj''
}


* DISPLAY RESULTS

display _newline as text "Test statistic: F( `r', " %8.0g `ddf' ") = " ///
    as result %8.3f `lr_test_stat'  _newline ///
    as text "Prob > F" as result %10.4f `pvalue'

end




* LOGIT LL
* 12sept2008
* now handles null models for h0
* 05sept2008
* logit likelihood function
* lnL = sum[for all y!=0](w_j lnF(x_j*b)) + sum[for all y==0](w_j ln{1-F(x_j*b)})
* F(z) = e^z +(1+e^z)
* w_j are optional weights

mata:
    void constrained_ll_logit(string scalar markvar1, string scalar markvar0, string scalar l_IVs,string scalar l_dep,string scalar betas)
    {
        B = st_matrix(betas)
        mat_yeq1 = st_data(.,tokens(l_IVs),markvar1)
        mat_yeq0 = st_data(.,tokens(l_IVs),markvar0)

        /* test that mat_yeq1 is a constant, indicating a null model */
        /* if not a null model, augment with 1's */
        if(mean(mat_yeq1)==1&variance(mat_yeq1)==0){

        }
        else{
            /* augment matrices of x_j's with 1's */
            mat_yeq1 = (mat_yeq1,J(rows(mat_yeq1),1,1))
            mat_yeq0 = (mat_yeq0,J(rows(mat_yeq0),1,1))
        }

        /* get x_j*b for y=1 and y=0 */
        mat_yeq1 = mat_yeq1*B'
        mat_yeq0 = mat_yeq0*B'

        /* get F(x_j*b) where F(z) = e^z +(1+e^z) */
        F_yeq1 = exp(mat_yeq1):/(1:+exp(mat_yeq1))
        F_yeq0 = exp(mat_yeq0):/(1:+exp(mat_yeq0))

        /* take the log */
        F_yeq1 = ln(F_yeq1)
        F_yeq0 = ln(1:-F_yeq0)
        ll = sum(F_yeq1)+sum(F_yeq0)
        st_numscalar("ll",ll)
    }
end

* l_dep - name of dv
* l_insample - name of variable marking the estimation sample
* l_mj - the imputation number
* l_IVs - list of IVs
* betas - name of the matrix containing the betas
* l_markvar - not actually used by constrained_ll_ologit, it creates its own markvars


program define constrained_ll_logit, rclass
    version 9
    syntax , l_insample(string) l_mj(real) [l_IVs(string)] ///
        l_dep(string) betas(string)

    if("`l_IVs'"==""){
        tempvar one
        gen `one' = 1
        local l_IVs `one'
    }
    tempvar markvar1
    tempvar markvar0
    quietly: gen `markvar1' = (`l_dep'!=0&`l_dep'!=.&`l_insample'==1&_mj==`l_mj')
    quietly: gen `markvar0' = (`l_dep'==0&`l_insample'==1&_mj==`l_mj')

    mata: constrained_ll_logit("`markvar1'","`markvar0'","`l_IVs'","`l_dep'","`betas'")
    return scalar ll = ll

end


* OLOGIT LL
* ram 11sept08
* lowest value              ln(1/(1+exp(-`a1' + `xb')))                          if $ML_y1 == 0
* intermediate values       ln(1/(1+exp(-`a2'+ `xb')) - 1/(1+exp(-`a1' + `xb'))) if $ML_y1 == 1
* highest value         ln(1 - 1/(1+exp(-`a3'+ `xb')))                       if $ML_y1 == 3

mata:
void constrained_ll_ologit(string scalar betas, real scalar cats, string scalar cuts, string scalar IVs, string scalar markvars)
{
    B = st_matrix(betas)
    if(rows(B)==0) {
        B = (0)
    }
    C = st_matrix(cuts)
    markvars = tokens(markvars)
    ll_vector = J(cats,1,.)
    a2 = 0
    for(i=1; i<=cats; i++) {

        X  = st_data(.,tokens(IVs),markvars[1,i])
        XB = X*B'

        /******** lowest value *********/
        if(i==1){
            a1 = C[1,i]
            pl = sum(ln(1:/(1:+exp(-a1:+XB))))
        }
        /* ******* all middle categories ******* */
        if(i>1&i<cats){
            a1 = C[1,i]
            pl = sum(ln(1:/(1:+exp(-a1:+XB)) :- 1:/(1:+exp(-a2:+XB))))
        }
        /* ******* highest value ******* */
        if(i==cats){
            pl=sum(ln(1 :- 1:/(1:+exp(-a2:+XB))))
        }
        ll_vector[i,1] = pl
        a2 = a1
    }
    st_numscalar("ll",sum(ll_vector))
}

end

* the ologit program
* l_dep - name of dv
* l_insample - name of variable marking the estimation sample
* l_mj - the imputation number
* l_IVs - list of IVs
* betas - name of the matrix containing the betas
* cats - number of categories of y
* cuts - name of matrix containing the cutpoints
* l_markvar - not actually used by constrained_ll_ologit, it creates its own markvars

program define constrained_ll_ologit, rclass
    version 9
    syntax , l_dep(string) l_insample(string) l_mj(real) [l_IVs(string)] ///
        betas(string) cats(real) cuts(string)


quietly: levelsof `l_dep', local(dep_levels)
local i = 1
local levels
foreach l of local dep_levels {
    local levels `levels' `l'
    tempvar markvar`i'
    gen `markvar`i'' = (`l_dep'==`l'&`l_insample'==1&_mj==`l_mj')
    local markvars `markvars' `markvar`i''
}
if("`l_IVs'"==""){
    tempvar zero
    gen `zero' = 0
    local l_IVs `zero'
}
    mata: constrained_ll_ologit("`betas'", `cats', "`cuts'", "`l_IVs'","`markvars'")
    return scalar ll = ll

end

* REG LL
* 08sept2008
* regression likelihood
* -(N/2)log(2pi)-(N/2)log(s^2)-(1/2s^2)(y-XB)'(y-XB)
* s = sigma = variance of XB
* y is y
* XB is the liniar prediction
* N is sample size
* pi is the value pi

mata:
void constrained_ll_reg(string scalar markvar,string scalar l_IVs,string scalar l_dep,string scalar  betas)
    {
        B = st_matrix(betas)
        X = st_data(.,tokens(l_IVs),markvar)
        Y = st_data(.,l_dep,markvar)
        if(mean(X)==1&variance(X)==0){

        }
        else{
            /* augment matrices of x_j's with 1's */
            X = (X,J(rows(X),1,1))
        }
        n = rows(X)
        XB = X*B'
        /* mata drop X */
        s2 = (Y-XB)'(Y-XB)/n
        ll =  -1*(n/2)*ln(2*pi()) - (n/2)*ln(s2) - (1/(2*s2))*(Y - XB)'*(Y - XB)
        st_numscalar("ll",ll)
    }
end

program define constrained_ll_reg, rclass
    version 9
    syntax , l_insample(string) l_mj(real) [l_IVs(string)] ///
        l_dep(string) betas(string)

if("`l_IVs'"==""){
    tempvar one
    gen `one' = 1
    local l_IVs `one'
}
tempvar markvar
quietly: gen `markvar' = (`l_insample'==1&_mj==`l_mj')
mata: constrained_ll_reg("`markvar'", "`l_IVs'", "`l_dep'", "`betas'")
return scalar ll = ll
end
